clear all
close all
clc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generate data  %%%%%%%%%%%%%%%%
%%%%%
N1=20;
x1=2+3*randn(1,N1);
y1=ones(1,N1);
N2=N1;
x2=-5+3*randn(1,N2);
y2=zeros(1,N2);
N=N1+N2;
xi=[x1 x2];
yi=[y1 y2];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p=@(x,mu,s) 1./(1+exp(-(x-mu)/s)); %%% definition logistic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% plot POSTERIOR  %%% IN THIS CASE (ONLY 2 PARAMETERS) CAN BE DONE 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mu_grid=-6:0.01:3;
s_grid=0.1:0.01:4.5;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
std_prior=10; %%%% parametro prior non usato per adesso
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:length(mu_grid)
    for j=1:length(s_grid)
        [logPost(i,j),logL(i,j)]=LogPost_eval(mu_grid(i),s_grid(j),xi,yi,std_prior);
    end
end
%%%%%%%%%%%%%%%%%%
figure
pcolor(s_grid,mu_grid,exp(logPost))
title('TRUE POSTERIOR - grid approximation')
xlabel('s')
ylabel('mu')
shading interp
movegui('west')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  METROPOLIS-HASTINGS ALGORITHM (in 2D)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% initializations %%%%%%%
mu_c=0; %%%% initial state for mu
s_c=0.5;  %%%% initial state for s
%T=100000;
T=10000; %%%% length of the generated MARKOV CHAIN
std_prop=0.8; %%%% std of the two Gaussian proposals (one Gaussian proposal in 2D)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for t=2:T+1
    mu_prop=mu_c(t-1)+std_prop*randn(1,1); %%%% GAUSSIAN RANDOM WALK PROPOSAL DENSITY FOR mu
    s_prop=s_c(t-1)+std_prop*randn(1,1);   %%%% GAUSSIAN RANDOM WALK PROPOSAL DENSITY FOR s 
    %%%%%%%%%%%%% BELOW WE USE A SIMPLIFIED VERSION SINCE
    %%%%%%%%%%%%%  THE GAUSSIAN RANDOM WALK PROPOSAL DENSITY IS SYMMETRIC
    logrho=LogPost_eval(mu_prop,s_prop,xi,yi,std_prior)-LogPost_eval(mu_c(t-1),s_c(t-1),xi,yi,std_prior); 
    a=min([1,exp(logrho)]); %%%% PROBABILITY OF ACCEPTING THE PROPOSED "JUMP"
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% just an additional check to avoid numerical problems
    if s_prop<0 %%%% logPost should be -inf and logrho -inf and a=0
        a=0; %%%% just to ensure that and avoiding "not a number" NaN 
    end
    %%%%%% TEST FOR ACCEPTING OR REJECTING THE PROPOSED NEW STATE %%%%%
    u=rand(1,1);
    if u<=a
        %%%%% we accept the jump
         mu_c(t)=mu_prop;
         s_c(t)=s_prop; 
    else
         %%%%% we DOES NOT accept the jump; we stay in the same previus state
         mu_c(t)=mu_c(t-1);
         s_c(t)=s_c(t-1); 
    end
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot(mu_c(1:100),'b-')
hold on
plot(s_c(1:100),'r-')
xlabel('t - iteration of the chain ')
title('First 100 samples generated by the MH algorithm')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 [a,b]=hist3([mu_c' s_c'],[40 40]);
 figure 
 pcolor(b{2},b{1},a)
 title(' POSTERIOR approximation via MH samples - shading interp ')
 xlabel('s')
ylabel('mu')
 shading interp
  movegui('center')
  %%%%%%%%%%%%%%%%%
  figure
 pcolor(b{2},b{1},a)
 title(' POSTERIOR approximation via MH samples - shading flat ')
 shading flat
 xlabel('s')
ylabel('mu')
 movegui('east')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% histogramas parametros %%%%
figure
hist(mu_c,30)
xlabel('mu')
figure
hist(s_c,30)
xlabel('s')
figure 
plotmatrix([mu_c' s_c']);

disp(' -------------------------------- ')
disp(' LOOK THIS FIRST FIGURES/PLOTS ')
disp(' THEN PRESS ANY LETTER/key ')
disp(' -------------------------------- ')
pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% FIRST POSSIBLE SOLUTION: MAP (point) ESTIMATOR of mu and s
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[pos1,pos2]=find(exp(logPost)==max(max(exp(logPost)))); %%%% here we can use this...but in general we will use the samples
disp('MAP point estimators of mu and s: ')
s_map=s_grid(pos2)
mu_map=mu_grid(pos1)
disp('MMSE point estimators of mu and s: ')
s_mmse=mean(s_c)
mu_mmse=mean(mu_c)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot(xi,yi,'ro')
xnow=-10:0.01:10;
hold on
plot(xnow,p(xnow,mu_map,s_map),'k-')
plot(xnow,p(xnow,mu_mmse,s_mmse),'b-')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
disp(' Other option: mean of the full Bayesian solution ')
for i=1:length(s_c)
    if mod(i,1000)==0
        disp(['Computing the full Bayes Solution for logistic function --- ',num2str(i/length(s_c)*100),'%'])
    end
     Logistic_functions_from_chain(i,:)=p(xnow,mu_c(i),s_c(i)); 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot(xi,yi,'ro')
hold on
Logist_mean_curve=mean(Logistic_functions_from_chain);
Logist_quantile_10=quantile(Logistic_functions_from_chain,0.1);
Logist_quantile_90=quantile(Logistic_functions_from_chain,0.9);
plot(xnow,Logist_mean_curve,'k-')
plot(xnow,Logist_quantile_10,'b--')
plot(xnow,Logist_quantile_90,'m--')
xlabel('x')
legend('data','MeanFullBayes','Quantile 10','Quantile 90','Location','West')
set(gca,'Fontsize',15)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure
plot(xnow,Logist_quantile_90-Logist_quantile_10,'k-','LineWidth',2)
title('Length of different credibible intervals at different xtest (more or less uncertainty)')
xlabel('x test')
ylabel('length of CI (degree of uncertainty)')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Uncertainty on the probability of beloging to a class (solution) at xtest =')
xnow(30)
figure
 hist(Logistic_functions_from_chain(:,30),30)
 title(['Uncertainty on the probability of beloging to a class at xtest = ',num2str(xnow(30))])
 xlabel('Probability of beloging to a class')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
disp('Uncertainty on the probability of beloging to a class (solution) at xtest =')
xnow(1000)
figure
 hist(Logistic_functions_from_chain(:,1000),20)
 title(['Uncertainty on the probability of beloging to a class at xtest = ',num2str(xnow(1000))])
  xlabel('Probability of beloging to a class')
  
  